@@ -4,7 +4,7 @@ import os
 import logging
 import logging.handlers
 import pandas as pd
-from datetime import datetime
+from datetime import datetime, timedelta
 import pytz
 import grpc
 from numerous_api_client.python_protos import spm_pb2_grpc, spm_pb2
@@ -117,8 +117,13 @@ class NumerousBufferedWriter:
                 try:
                     self._write_method(write_generator(), must_flush=must_flush)
                     not_done = False
-                except Exception as e:
-                    print(e)
+                except:
+                    tb = traceback.format_exc()
+                    log.error(tb)
+                    raise KeyboardInterrupt('Writer failed')
+
+
+
 
         self.writer_thread = threading.Thread(target=writer_thread_func)
         self.writer_thread.start()
@@ -161,9 +166,24 @@ class NumerousBufferedWriter:
 class NumerousLogHandler:
     def __init__(self, client):
         self.client = client
+        self.buffered_records = []
+        self.last_push = datetime.utcnow() - timedelta(hours=1)
+        self.debounce = 5
 
     def handle(self, record):
-        self.client.push_scenario_log(record.msg)
+        now_ = datetime.utcnow()
+        #print(record)
+        self.buffered_records.append((now_, record.msg))
+
+        if (now_ - self.last_push).seconds > self.debounce:
+            self.last_push = now_
+            self.flush()
+
+
+    def flush(self):
+        #print('recs: ', self.buffered_records)
+        self.client.push_scenario_logs(self.buffered_records)
+        self.buffered_records=[]
 
     # TODO make a handler thread to schedule update to server!
 
@@ -251,8 +271,8 @@ class NumerousClient:
         logger = logging.getLogger()
         logger.setLevel(logging.ERROR)
         logger.addHandler(numerous_logger)
-
-        self.log_listener = logging.handlers.QueueListener(log_queue, NumerousLogHandler(self))
+        self.log_handler = NumerousLogHandler(self)
+        self.log_listener = logging.handlers.QueueListener(log_queue, self.log_handler)
         self.log_listener.start()
 
     def _refresh_access_token(self, refresh_token, instance_id, execution_id):
@@ -359,7 +379,7 @@ class NumerousClient:
 
             self.last_progress=tic
             self._base_client.SetScenarioProgress(spm_pb2.ScenarioProgress(
-                project=self._project, scenario=scenario, spm_job_id=self._job_id,
+                project=self._project, scenario=scenario, job_id=self._job_id,
                 message=message, status=status, clean=clean, progress=progress
             ))
 
@@ -400,9 +420,13 @@ class NumerousClient:
 
     def push_scenario_logs(self, logs):
 
+        #print('logs: ',logs)
+        timestamps = [l[0].timestamp() for l in logs]
+        #print('time: ', timestamps)
         self._base_client.PushExecutionLogEntries(spm_pb2.LogEntries(
+            scenario=self._scenario,
             execution_id=self._execution_id, log_entries=[l[1] for l in logs],
-            timestamps=[l[0].timestamp() for l in logs]
+            timestamps=timestamps
         ))
 
     def delete_scenario(self, scenario=None):
@@ -435,7 +459,7 @@ class NumerousClient:
 
         if file is None:
             file = local_file.split('/')[-1]
-        print('File: ', file)
+        #print('File: ', file)
         upload_url = self._base_client.GenerateScenarioUploadSignedURL(spm_pb2.ScenarioFilePath(project=self._project, scenario=scenario, path=file, file_id=file_id))
 
         def upload():
@@ -695,9 +719,11 @@ class NumerousClient:
 
             #if len(rows)>0:
             blocks = self._rows_to_blocks(rows)
+
+
             yield spm_pb2.DataList(scenario=scenario, data=blocks, clear=clear, block_complete=True, row_complete=True)
 
-            return
+            #return
 
         self._base_client.WriteDataList(gen(clear))
 
@@ -801,14 +827,15 @@ class NumerousClient:
     def publish_message(self, message):
         channel = ".".join(['EVENT', self._project, self._scenario])
 
-        self._base_client.PublishSubcsriptionMessage(spm_pb2.SubscriptionMessage(channel=channel, message=json.dumps(message)))
+        self._base_client.PublishSubcsriptionMessage(spm_pb2.SubscriptionMessage(channel=channel, message=json.dumps(message)), project_id=self._project, scenario_id=self._scenario)
     #def subscribe_channels(self, channels):
     #    self._base_client.SubscribeForUpdates(spm_pb2.)
     def subscribe_messages(self, channel_patterns):
-        for message in self._base_client.SubscribeForUpdates(spm_pb2.Subscription(channel_patterns=channel_patterns)):
+        for message in self._base_client.SubscribeForUpdates(spm_pb2.Subscription(channel_patterns=channel_patterns, scenario=self._scenario)):
             yield message.channel, json.loads(message.message)
 
     def refresh_token(self, refresh_token):
+        #print('!')
         refreshed_token = self._token_manager.GetAccessToken(spm_pb2.Token(val=refresh_token))
         return refreshed_token
 
@@ -822,6 +849,10 @@ class NumerousClient:
 
             self.complete_execution()
 
+        if hasattr(self, 'log_handler'):
+            log.debug('flushing log')
+            self.log_handler.flush()
+
         log.debug('Client closing')
         if hasattr(self,'log_listener'):
             self.log_listener.stop()
@@ -1053,8 +1084,11 @@ class NumerousAdminClient(NumerousClient):
 
         return spm_pb2.Empty()
 
-    def read_execution_logs(self):
-        for l in self._base_client.ReadExecutionLogEntries(spm_pb2.ExecutionReadLogs(execution_id=self._execution_id, start=0.0, end=0.0)):
+    def read_execution_logs(self, execution=None):
+        if execution is None:
+            execution=self._execution_id
+
+        for l in self._base_client.ReadExecutionLogEntries(spm_pb2.ExecutionReadLogs(execution_id=execution, start=0, end=0)):
             yield l.log_entry, datetime.fromtimestamp(l.timestamp)
 
 
